from django.contrib.auth import get_user_model
from rest_framework import serializers

from huscy.appointments import models, services


User = get_user_model()


class ParticipantSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.Invitation
        fields = (
            "participant",
        )


class AppointmentSerializer(serializers.ModelSerializer):
    creator = serializers.HiddenField(default=serializers.CurrentUserDefault())
    owner = serializers.PrimaryKeyRelatedField(queryset=User.objects.all(), required=False)
    participants = serializers.ListField(child=serializers.CharField(), required=False)

    class Meta:
        model = models.Appointment
        fields = (
            'creator',
            "description",
            "end",
            "id",
            "owner",
            "participants",
            "resource",
            "start",
            "title",
        )

    def to_representation(self, appointment):
        response = super().to_representation(appointment)
        response['participants'] = services.get_participants(appointment)
        return response

    def create(self, validated_data):
        validated_data.pop('owner', None)
        try:
            return services.create_appointment(**validated_data)
        except Exception as e:
            raise serializers.ValidationError(str(e))

    def update(self, appointment, validated_data):
        validated_data.pop('creator', None)
        if 'participants' not in validated_data:
            validated_data['participants'] = []
        try:
            return services.update_appointment(appointment=appointment, **validated_data)
        except Exception as e:
            raise serializers.ValidationError(str(e))


class InvitationSerializer(serializers.ModelSerializer):

    class Meta:
        model = models.Invitation
        fields = (
            "appointment",
            "id",
            "participant",
            "status",
        )


class ResourceSerializer(serializers.ModelSerializer):

    class Meta:
        model = models.Resource
        fields = (
            "id",
            "name",
        )



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('appointments', '0003_auto_20200226_0756'),
    ]

    operations = [
        migrations.AlterField(
            model_name='appointment',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='invitation',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='reminder',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='resource',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='token',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



from django.urls import include, path
from rest_framework.routers import DefaultRouter

from huscy.appointments import views
from huscy.appointments.feed import AppointmentFeed
from huscy.appointments.feed import get_feed_url


router = DefaultRouter()
router.register('appointments', views.AppointmentsViewSet, basename='appointment')
router.register('resources', views.ResourcesViewSet)


urlpatterns = [
    path('api/', include(router.urls)),

    path('appointments/feed/<str:token>', AppointmentFeed(), name='feed'),
    path('appointments/feedurl/', get_feed_url, name='feed-url'),
]



from django_filters import rest_framework as filters

from huscy.appointments.models import Appointment


class AppointmentFilter(filters.FilterSet):
    # Note: from_time and to_time express a time range within the appointment starts
    from_time = filters.DateTimeFilter(field_name="start", lookup_expr='gte')
    to_time = filters.DateTimeFilter(field_name="start", lookup_expr='lte')

    class Meta:
        model = Appointment
        fields = (
            'from_time',
            'owner',
            'resource',
            'title',
            'to_time',
        )



import socket

from django_ical.views import ICalFeed
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from huscy.appointments import models, services


@api_view(['GET'])
@permission_classes([IsAuthenticated])
def get_feed_url(request):
    token, _ = models.Token.objects.get_or_create(user=request.user)
    return Response({'feed': f'/feed/{token.key}'})


class AppointmentFeed(ICalFeed):
    timezone = 'UTC'
    file_name = "appointments.ics"

    def __call__(self, request, *args, **kwargs):
        try:
            self.user = models.Token.objects.get(key=kwargs['token']).user
        except models.Token.DoesNotExist:
            self.user = None

        return super(AppointmentFeed, self).__call__(request, *args, **kwargs)

    def items(self):
        if self.user and self.user.is_superuser:
            return services.get_appointments()
        elif self.user:
            return services.get_appointments(owner=self.user)
        return []

    def item_guid(self, item):
        return f'{item.id}@{socket.getfqdn()}'

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.description

    def item_start_datetime(self, item):
        return item.start

    def item_end_datetime(self, item):
        return item.end

    def item_link(self, item):
        return ''



# Generated by Django 3.0.2 on 2020-01-31 06:18

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import huscy.appointments.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Appointment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, default='New Appointment', max_length=255)),
                ('description', models.TextField(blank=True, default='')),
                ('start', models.DateTimeField()),
                ('end', models.DateTimeField()),
                ('resource', models.CharField(blank=True, default='', max_length=128)),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Resource',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128)),
            ],
        ),
        migrations.CreateModel(
            name='Token',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(default=huscy.appointments.models.create_token, max_length=64, unique=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Reminder',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('remind_at', models.DateTimeField()),
                ('appointment', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='reminders', to='appointments.Appointment')),
            ],
        ),
        migrations.CreateModel(
            name='Invitation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('participant', models.CharField(max_length=128)),
                ('status', models.PositiveSmallIntegerField(choices=[(0, 'pending'), (1, 'accepted'), (2, 'decliend')], default=0)),
                ('appointment', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='invitations', to='appointments.Appointment')),
            ],
        ),
    ]
